@@ -17,7 +17,7 @@
 import os
 import logging
 
-from gi.repository import GObject
+from gi.repository import GLib
 from gi.repository import Gdk
 from gi.repository import SugarExt
 
@@ -101,7 +101,7 @@ class KeyHandler(object):
                 except Exception:
                     logging.exception('Exception while loading extension:')
 
-        self._key_grabber.grab_keys(_actions_table.keys())
+        self._key_grabber.grab_keys(list(_actions_table.keys()))
 
     def _change_volume(self, step=None, value=None):
         if step is not None:
@@ -164,7 +164,7 @@ class KeyHandler(object):
         if "SUGAR_DEVELOPER" in os.environ:
             session_manager = session.get_session_manager()
             session_manager.logout()
-            GObject.timeout_add_seconds(3, session_manager.shutdown_completed)
+            GLib.timeout_add_seconds(3, session_manager.shutdown_completed)
 
     def handle_open_search(self, event_time):
         journalactivity.get_journal().show_journal()
@@ -193,7 +193,7 @@ class KeyHandler(object):
         panel.show()
 
     def handle_dump_ui_tree(self, event_time):
-        print uitree.get_root().dump()
+        print(uitree.get_root().dump())
 
     def _key_pressed_cb(self, grabber, keycode, state, event_time):
         key = grabber.get_key(keycode, state)
@@ -221,7 +221,7 @@ class KeyHandler(object):
 
             if hasattr(action, 'handle_key_press'):
                 action.handle_key_press(key)
-            elif isinstance(action, basestring):
+            elif isinstance(action, str):
                 method = getattr(self, 'handle_' + action)
                 method(event_time)
             else:
