@@ -25,6 +25,7 @@ from gettext import gettext as _
 import gi
 gi.require_version('GtkSource', '3.0')
 from gi.repository import GObject
+from gi.repository import GLib
 from gi.repository import Pango
 from gi.repository import Gtk
 from gi.repository import Gdk
@@ -334,6 +335,7 @@ class ViewSource(Gtk.Window):
             os.unlink(document_path)
 
         self._gdk_window.set_cursor(Gdk.Cursor(Gdk.CursorType.LEFT_PTR))
+        Gdk.flush()
 
     def __key_press_event_cb(self, window, event):
         keyname = Gdk.keyval_name(event.keyval)
@@ -365,7 +367,7 @@ class DocumentButton(RadioToolButton):
 
         self.props.tooltip = _('Instance Source')
 
-        settings = Gio.Settings('org.sugarlabs.user')
+        settings = Gio.Settings.new('org.sugarlabs.user')
         self._color = settings.get_string('color')
         icon = Icon(file=file_name,
                     pixel_size=style.STANDARD_ICON_SIZE,
@@ -410,7 +412,7 @@ class DocumentButton(RadioToolButton):
             new_alert.props.title = _("Duplicating activity...")
 
             self.get_toplevel().add_alert(new_alert)
-            GObject.idle_add(internal_callback, new_alert)
+            GLib.idle_add(internal_callback, new_alert)
 
     def __set_busy_cursor(self, busy):
         cursor = None
@@ -422,6 +424,7 @@ class DocumentButton(RadioToolButton):
 
         gdk_window = self.get_root_window()
         gdk_window.set_cursor(cursor)
+        Gdk.flush()
 
     def __copy_to_home_cb(self, menu_item, copy_alert=None):
         """Make a local copy of the activity bundle in user_activities_path"""
@@ -453,7 +456,7 @@ class DocumentButton(RadioToolButton):
                 finally:
                     self.__set_busy_cursor(False)
 
-            GObject.idle_add(async_copy_activity_tree)
+            GLib.idle_add(async_copy_activity_tree)
         else:
             if copy_alert:
                 self.get_toplevel().remove_alert(copy_alert)
